import logging

import pytest

from EscribaLogger.Log import Log


@pytest.fixture
def log_obj():
    return Log()


def test_log_should_be_a_singleton(log_obj: Log):
    assert log_obj == Log()


def test_log_can_change_name():
    default_logger_name = Log.log_name()
    new_logger_name = "my_logger_test"
    Log.set_logger_name(new_logger_name)
    assert Log.log_name() == new_logger_name and Log.log_name() != default_logger_name


def test_log_can_add_drivers():
    number_of_default_drivers = len(Log.root_logger.handlers)
    Log.add_driver("stdout")
    number_of_current_drivers = len(Log.root_logger.handlers)
    Log.clear_drivers()
    assert number_of_current_drivers == 1 and number_of_default_drivers == 0


def test_log_cannot_add_same_driver_twice():
    number_of_default_drivers = len(Log.root_logger.handlers)
    Log.add_driver("stdout")
    Log.add_driver("stdout")
    number_of_current_drivers = len(Log.root_logger.handlers)
    Log.clear_drivers()
    assert number_of_current_drivers == 1 and number_of_default_drivers == 0


def test_log_cannot_add_unexistent_driver():
    with pytest.raises(Exception):
        Log.add_driver("BadDriver")


def test_log_can_add_generic_driver():
    class CustomGenericHandler(logging.Handler):
        ...

    Log.add_driver(
        "newDriver", driver_func=lambda driver_option: CustomGenericHandler()
    )

    log_handlers = Log.root_logger.handlers
    assert (
        type(log_handlers[0]).__name__ == "CustomGenericHandler"
        and len(log_handlers) == 1
    )


def test_log_can_remove_generic_driver():
    class CustomGenericHandler(logging.Handler):
        ...

    Log.add_driver(
        "newDriver", driver_func=lambda driver_option: CustomGenericHandler()
    )

    Log.add_driver("file")
    Log.remove_driver("file")

    stream_type = Log.drivers["newDriver"](None)
    Log.remove_driver("newDriver")
    log_drivers = Log.drivers.keys()
    current_drivers = Log.root_logger.handlers
    assert len(current_drivers) == 0


def test_log_can_right_info(mocker):
    mocker.patch("logging.Logger.info")
    Log.info("message")
    assert logging.Logger.info.assert_called_with("message", extra=None) is None

    custom_extra = {"custom": "value"}
    Log.info("another message", extra=custom_extra)
    assert (
        logging.Logger.info.assert_called_with("another message", extra=custom_extra)
        is None
    )


def test_log_can_right_warning(mocker):
    mocker.patch("logging.Logger.warning")
    Log.warn("message")
    Log.warning("message")
    assert logging.Logger.warning.assert_called_with("message", extra=None) is None

    custom_extra = {"custom": "value"}
    Log.warn("another message", extra=custom_extra)
    Log.warning("another message", extra=custom_extra)
    assert (
        logging.Logger.warning.assert_called_with("another message", extra=custom_extra)
        is None
    )


def test_log_can_right_error(mocker):
    mocker.patch("logging.Logger.error")
    Log.error("message")
    assert logging.Logger.error.assert_called_with("message", extra=None) is None

    custom_extra = {"custom": "value"}
    Log.error("another message", extra=custom_extra)
    assert (
        logging.Logger.error.assert_called_with("another message", extra=custom_extra)
        is None
    )


def test_log_can_right_critical(mocker):
    mocker.patch("logging.Logger.critical")
    Log.critical("message")
    assert logging.Logger.critical.assert_called_with("message", extra=None) is None

    custom_extra = {"custom": "value"}
    Log.critical("another message", extra=custom_extra)
    assert (
        logging.Logger.critical.assert_called_with(
            "another message", extra=custom_extra
        )
        is None
    )



import logging
import os
from datetime import datetime
from typing import Optional, TypedDict

import graypy
from rich.highlighter import Highlighter
from rich.logging import RichHandler
from rich.text import Text


class DriverOption(TypedDict):
    file_location: Optional[str]
    graylog_host: Optional[str]
    graylog_port: Optional[int]
    graylog_protocol: Optional[str]


def driver_file(driver_option: DriverOption = None):
    if not driver_option:
        driver_option = {"file_location": "logs"}

    formatter_string = "[%(asctime)s] "
    formatter_string += "%(name)s.%(levelname)s"
    formatter_string += " - %(message)s"
    formatter_string += "%(extra_context)s"

    formatter = logging.Formatter(formatter_string)
    # formatter.default_time_format = '%Y-%m-%d %H:%M:%s'

    log_file_name = datetime.now().strftime("%Y-%m-%d.log")
    log_file_location = driver_option.get("file_location", "logs")

    log_file_path = os.path.join(log_file_location, log_file_name)

    stream = logging.FileHandler(log_file_path)
    stream.setFormatter(formatter)
    return stream


def driver_stdout(driver_option: DriverOption = None):
    class LogNameHighlighter(Highlighter):
        def highlight(self, text: Text) -> None:
            text.highlight_regex(r"^\w+ - ", style="black italic")

    rich_handler = RichHandler(
        highlighter=LogNameHighlighter(),
        level=logging.DEBUG,
        omit_repeated_times=False,
        tracebacks_show_locals=True,
        tracebacks_extra_lines=True,
        tracebacks_word_wrap=True,
        rich_tracebacks=True,
    )

    formatter_string = "%(name)s - %(message)s"

    formatter = logging.Formatter(formatter_string)
    rich_handler.setFormatter(formatter)

    return rich_handler


def driver_graylog(driver_options: DriverOption = None):
    graylog_host = driver_options.get("graylog_host", "localhost")
    graylog_port = driver_options.get("graylog_port", 12201)

    formatter_string = "%(name)s.%(levelname)s - %(message)s"
    formatter = logging.Formatter(formatter_string)
    stream = graypy.GELFHTTPHandler(graylog_host, graylog_port)
    stream.setFormatter(formatter)
    return stream



import pytest

from EscribaLogger.process_extra_context import process_extra_context


@pytest.mark.parametrize(
    "extra, expected",
    [
        (None, ""),
        ({"item": "value"}, r' - {"item": "value"}'),
    ],
)
def test_extra_context_processing_return_right_value(extra, expected):
    assert process_extra_context(extra) == expected


def test_extra_context_processing_raises_type_error_for_non_dict_parameters():
    with pytest.raises(TypeError):
        process_extra_context("some bad string")



from logging import FileHandler

import pytest
from rich.logging import RichHandler

from EscribaLogger.drivers import DriverOption, driver_file, driver_stdout


@pytest.fixture
def stdout_stream():
    return driver_stdout()


@pytest.fixture
def file_stream():
    options: DriverOption = {"file_location": "logs"}
    return driver_file(options)


def test_driver_stdout_should_be_a_stream(stdout_stream: RichHandler):
    assert isinstance(stdout_stream, RichHandler)


def test_driver_stdout_should_init_in_debug_mode(stdout_stream: RichHandler):
    stream_level = stdout_stream.level
    debug_code = 10
    assert stream_level == debug_code


def test_driver_file_should_be_a_stream(file_stream: FileHandler):
    assert isinstance(file_stream, FileHandler)


def test_driver_file_should_init_in_debug_mode(file_stream: FileHandler):
    stream_level = file_stream.level
    debug_code = 0
    assert stream_level == debug_code



import json


def process_extra_context(context: dict):
    if not context:
        return ""
    if type(context) is str:
        raise TypeError(
            f"Context must be a object dict, not {type(context).__name__}!!!"
        )
    return " - " + json.dumps(context)



import logging

from .DPSingleton import DPSingleton
from .drivers import DriverOption, driver_file, driver_graylog, driver_stdout
from .process_extra_context import process_extra_context


class Log(metaclass=DPSingleton):
    root_logger: logging.Logger = logging.getLogger("EscribaLogger")
    drivers = {"file": driver_file, "stdout": driver_stdout, "graylog": driver_graylog}

    def __init__(self, log_name: str = None) -> None:
        self.root_logger.name = log_name
        self.root_logger.setLevel(logging.DEBUG)
        self.add_filter("extra_context", None)

    @staticmethod
    def handle_any_exception(exc_type, exc_value, exc_traceback):
        Log.root_logger.setLevel(logging.DEBUG)
        Log.root_logger.exception(
            "Uncaught exception\n",
            exc_info=(exc_type, exc_value, exc_traceback),
        )

    @staticmethod
    def log_name() -> str:
        return Log.root_logger.name

    @staticmethod
    def set_logger_name(name: str):
        Log.root_logger.name = name

    @staticmethod
    def add_filter(filter_name: str, value: str):
        def define_filter(record: logging.LogRecord):
            setattr(record, filter_name, value)
            return True

        Log.root_logger.addFilter(define_filter)

    @staticmethod
    def add_driver(
        driver_name: str = "stdout",
        driver_func: callable = None,
        driver_options: DriverOption = None,
    ):
        if driver_func:
            Log.drivers[driver_name] = driver_func
        elif driver_name not in Log.drivers:
            raise Exception("Driver not exists")

        current_handlers_list = list(
            map(
                lambda handler: type(handler).__name__,
                Log.root_logger.handlers,
            )
        )
        stream = Log.drivers[driver_name](driver_options)
        if type(stream).__name__ in current_handlers_list:
            return

        Log.root_logger.addHandler(stream)

    @staticmethod
    def remove_driver(driver_name: str):
        if not driver_name:
            raise ValueError("You must specify a valid driver name!!!")
        if driver_name not in Log.drivers.keys():
            raise ValueError(
                f'Driver "{driver_name}" not found. Available drivers: {str(Log.drivers.keys())}!!!'
            )

        Log.root_logger.handlers = list(
            filter(
                lambda handler: type(handler).__name__
                != type(Log.drivers[driver_name](driver_option={})).__name__,
                Log.root_logger.handlers,
            )
        )
        del Log.drivers[driver_name]

    @staticmethod
    def clear_drivers():
        Log.root_logger.handlers = []

    @staticmethod
    def info(msg: str, extra: dict = None):
        Log.add_filter("extra_context", process_extra_context(extra))
        Log.root_logger.info(msg, extra=extra)

    @staticmethod
    def warning(msg: str, extra: dict = None):
        Log.add_filter("extra_context", process_extra_context(extra))
        Log.root_logger.warning(msg, extra=extra)

    @staticmethod
    def warn(msg: str, extra: dict = None):
        Log.root_logger.warning(msg, extra=extra)

    @staticmethod
    def error(msg: str, extra: dict = None):
        Log.add_filter("extra_context", process_extra_context(extra))
        Log.root_logger.error(msg, extra=extra)

    @staticmethod
    def critical(msg: str, extra: dict = None):
        Log.add_filter("extra_context", process_extra_context(extra))
        Log.root_logger.critical(msg, extra=extra)
